@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
 
-__version__ = "2.2.0"
+__version__ = "2.3.1"
 __name__ = "brawlpython"
 
 
+from .api import API
 from .clients import AsyncClient, SyncClient
 
-
 __all__ = (
     "AsyncClient",
     "SyncClient",
+    "API"
 )
