@@ -7,7 +7,7 @@ from collections import defaultdict
 from functools import update_wrapper
 from requests import Session
 
-from .api_toolkit import make_headers, multiparams
+from .api_toolkit import default_headers, multiparams, isiter_noliterals, ismapping
 from .base_classes import AsyncInitObject, AsyncWith, SyncWith
 from .cache_utils import somecachedmethod, iscorofunc
 from .exceptions import WITH_CODE, UnexpectedResponseCode
@@ -46,9 +46,11 @@ def _raise_for_status(self, url: str, code: int,
                       data: Union[Mapping[str, Any], str]) -> None:
 
     if isinstance(data, str):
-        reason, message = "", data
+        reason = "without a reason"
+        message = data if len(data) > 0 else "no message"
     else:
-        reason, message = data.get("reason", ""), data.get("message", "")
+        reason = data.get("reason", "without a reason")
+        message = data.get("message", "no message")
 
     excp = next(filter(lambda x: x.code == code, WITH_CODE), None)
     if excp is not None:
@@ -146,7 +148,7 @@ def retry_to_get_data(func):
                     elif i == 0:
                         self.raise_for_status(url, code, data)
                     else:
-                        for key, val in kw:
+                        for key, val in kw.items():
                             d_kwargs[key].append(val)
 
                         for i, val in enumerate(a):
@@ -202,14 +204,24 @@ def retry_to_get_data(func):
     return update_wrapper(wrapper, func)
 
 
+def headers_handler(self, headers):
+    if not self.can_use_cache:
+        return headers
+
+    if isiter_noliterals(headers) and not ismapping(headers):
+        return [tuple(zip(headers.items())) for header in headers]
+    else:
+        return tuple(zip(headers.items()))
+
+
 class AsyncSession(AsyncInitObject, AsyncWith):
-    async def __init__(self, token: str, trust_env: bool = True,
+    async def __init__(self, trust_env: bool = True,
                        cache_ttl: Union[int, float] = 60,
                        cache_limit: int = 1024,
                        use_cache: bool = True,
                        timeout: Union[int, float] = 30,
                        repeat_failed: int = 3) -> None:
-        headers = make_headers(token)
+        headers = default_headers()
         loop = asyncio.get_event_loop()
         self.session = ClientSession(
             loop=loop,
@@ -223,7 +235,7 @@ class AsyncSession(AsyncInitObject, AsyncWith):
             self.cache = TTLCache(maxsize=cache_limit, ttl=cache_ttl)
         else:
             self.cache = None
-        self.use_cache = use_cache
+        self._use_cache = use_cache
 
         if repeat_failed > 1:
             self._attempts = range(repeat_failed - 1, -1, -1)
@@ -255,11 +267,12 @@ class AsyncSession(AsyncInitObject, AsyncWith):
 
     @property
     def can_use_cache(self) -> bool:
-        return self.use_cache and isinstance(self.cache, TTLCache)
+        return self._use_cache and isinstance(self.cache, TTLCache)
 
-    async def _simple_get(self, url: str,
-                          from_json: bool = True) -> Tuple[int, str]:
-        async with self.session.get(url) as response:
+    async def _simple_get(
+            self, url: str, from_json: bool = True,
+            headers: Iterable[Iterable[str]] = {}) -> Tuple[int, str]:
+        async with self.session.get(url, headers=dict(headers)) as response:
             code = response.status
             data = await response.text()
             if from_json:
@@ -270,22 +283,29 @@ class AsyncSession(AsyncInitObject, AsyncWith):
     _get = retry_to_get_data(mix_all_gets(False)(_simple_get))
     _gets = retry_to_get_data(mix_all_gets(True)(_simple_get))
 
-    async def get(self, url: str, from_json: bool = True) -> R:
-        return (await self._get(url, from_json))[0]
+    async def get(self, url: str, from_json: bool = True,
+                  headers: Dict[str, str] = {}) -> R:
+        return (await self._get(
+            url, from_json=from_json,
+            headers=headers_handler(self, headers)))[0]
 
-    async def gets(self, urls: URLS, from_jsons: bool = True) -> L:
-        return await self._gets(urls, from_jsons)
+    async def gets(
+            self, urls: URLS, from_jsons: bool = True,
+            headers: Union[List[Dict[str, str]], Dict[str, str]] = {}) -> L:
+        return await self._gets(
+            urls, from_json=from_jsons,
+            headers=headers_handler(self, headers))
 
 
 class SyncSession(SyncWith):
-    def __init__(self, token: str, trust_env: bool = True,
+    def __init__(self, trust_env: bool = True,
                  cache_ttl: Union[int, float] = 60,
                  cache_limit: int = 1024, use_cache: bool = True,
                  timeout: Union[int, float] = 30,
                  repeat_failed: int = 3) -> None:
         self._closed = False
 
-        headers = make_headers(token)
+        headers = default_headers()
         self.session = Session()
         self.session.trust_env = trust_env
         self.session.headers.update(headers)
@@ -294,7 +314,7 @@ class SyncSession(SyncWith):
             self.cache = TTLCache(maxsize=cache_limit, ttl=cache_ttl)
         else:
             self.cache = None
-        self.use_cache = use_cache
+        self._use_cache = use_cache
 
         self.timeout = timeout
 
@@ -323,10 +343,13 @@ class SyncSession(SyncWith):
 
     @property
     def can_use_cache(self) -> bool:
-        return self.use_cache and isinstance(self.cache, TTLCache)
+        return self._use_cache and isinstance(self.cache, TTLCache)
 
-    def _simple_get(self, url: str, from_json: bool = True) -> Tuple[int, str]:
-        with self.session.get(url, timeout=self.timeout) as response:
+    def _simple_get(
+            self, url: str, from_json: bool = True,
+            headers: Iterable[Iterable[str]] = {}) -> Tuple[int, str]:
+        with self.session.get(
+                url, timeout=self.timeout, headers=dict(headers)) as response:
             code = response.status_code
             data = response.text
             if from_json:
@@ -337,8 +360,12 @@ class SyncSession(SyncWith):
     _get = retry_to_get_data(mix_all_gets(False)(_simple_get))
     _gets = retry_to_get_data(mix_all_gets(True)(_simple_get))
 
-    def get(self, url: str, from_json: bool = True) -> R:
-        return self._get(url, from_json)[0]
+    def get(self, url: str, from_json: bool = True,
+            headers: Dict[str, str] = {}) -> R:
+        return self._get(url, from_json=from_json,
+                         headers=headers_handler(self, headers))[0]
 
-    def gets(self, urls: URLS, from_jsons: bool = True) -> L:
-        return self._gets(urls, from_jsons)
+    def gets(self, urls: URLS, from_jsons: bool = True,
+             headers: Union[List[Dict[str, str]], Dict[str, str]] = {}) -> L:
+        return self._gets(urls, from_json=from_jsons,
+                          headers=headers_handler(self, headers))
